@@ -2,10 +2,9 @@
 
 likRatioFit <- function(outMod1, outMod2, dat1Mod1, dat1Mod2, dat2Mod1, dat2Mod2, 
     usedFit = NULL, prior = 1) {
-    if (is.null(usedFit)) 
-        usedFit <- getKeywords()$usedFit
+	usedFit <- cleanUsedFit(usedFit, colnames(dat1Mod1@fit), colnames(dat1Mod2@fit), colnames(dat2Mod1@fit), colnames(dat2Mod2@fit))
     
-    observedFit <- as.data.frame(rbind(extractLavaanFit(outMod1), extractLavaanFit(outMod2))[, 
+    observedFit <- as.data.frame(rbind(inspect(outMod1, "fit"), inspect(outMod2, "fit"))[, 
         usedFit])
     
     mod1 <- clean(dat1Mod1, dat1Mod2, dat2Mod1, dat2Mod2)
