@@ -9,11 +9,14 @@ plotDist <- function(object, xlim = NULL, ylim = NULL, r = 0, var = NULL, contou
         object <- extractSimDataDist(object, var)
     }
     if (object@p == 1) {
-        plotDist1D(object@margins[1], object@paramMargins[[1]], reverse = object@reverse[1], 
-            xlim = xlim)
+		if(any(is.na(object@skewness))) {
+			plotDist1D(object@margins[1], object@paramMargins[[1]], reverse = object@reverse[1], 
+				xlim = xlim)
+		} else {
+			plot(density(dataGen(object, n = 100000, m = matrix(0), cm = matrix(1))), main = paste0("Density: skewness = ", object@skewness, ", kurtosis = ", object@kurtosis))
+		}
     } else if (object@p == 2) {
-        plotDist2D(object@margins[1:2], object@paramMargins[1:2], reverse = object@reverse[1:2], 
-            xlim = xlim, ylim = ylim, r = r, contour = contour, cop = object@copula)
+        plotDist2D(object, xlim = xlim, ylim = ylim, r = r, contour = contour)
     } else {
         stop("The dimension cannot be greater than 2.")
     }
@@ -42,25 +45,24 @@ plotDist1D <- function(distName, param, xlim = NULL, reverse = FALSE) {
     lines(xrange, yrange)
 }
 
-plotDist2D <- function(distName, param, xlim = NULL, ylim = NULL, r = 0, reverse = rep(FALSE, 
-    2), contour = TRUE, cop = NULL) {
+plotDist2D <- function(object, xlim = NULL, ylim = NULL, r = 0, contour = TRUE) {
     library(copula)
-	if(!is.null(cop) && is(cop, "NullCopula")) {
+	if(any(is.na(object@skewness)) && !is.null(object@copula) && is(object@copula, "NullCopula")) {
 		CopNorm <- ellipCopula(family = "normal", dim = 2, dispstr = "un", param = r)
-		Mvdc <- mvdc(CopNorm, distName, param)
+		Mvdc <- mvdc(CopNorm, object@margins, object@paramMargins)
 
 		######################### xlim
 		if (is.null(xlim)) {
-			xfunmin <- c(list(get(paste("q", distName[1], sep = "")), 0.005), param[[1]])
-			xfunmax <- c(list(get(paste("q", distName[1], sep = "")), 0.995), param[[1]])
+			xfunmin <- c(list(get(paste("q", object@margins[1], sep = "")), 0.005), object@paramMargins[[1]])
+			xfunmax <- c(list(get(paste("q", object@margins[1], sep = "")), 0.995), object@paramMargins[[1]])
 			xlim <- rep(0, 0)
 			xlim[1] <- eval(as.call(xfunmin))
 			xlim[2] <- eval(as.call(xfunmax))
 		}
 		######################### ylim
 		if (is.null(ylim)) {
-			yfunmin <- c(list(get(paste("q", distName[2], sep = "")), 0.005), param[[2]])
-			yfunmax <- c(list(get(paste("q", distName[2], sep = "")), 0.995), param[[2]])
+			yfunmin <- c(list(get(paste("q", object@margins[2], sep = "")), 0.005), object@paramMargins[[2]])
+			yfunmax <- c(list(get(paste("q", object@margins[2], sep = "")), 0.995), object@paramMargins[[2]])
 			ylim <- rep(0, 0)
 			ylim[1] <- eval(as.call(yfunmin))
 			ylim[2] <- eval(as.call(yfunmax))
@@ -70,8 +72,12 @@ plotDist2D <- function(distName, param, xlim = NULL, ylim = NULL, r = 0, reverse
 		grids <- as.matrix(expand.grid(xis, yis))
 		zmat <- matrix(dMvdc(grids, Mvdc), 51, 51)
 	} else {
-		Mvdc <- mvdc(cop, distName, param)
-		Data <- CopSEM(Mvdc, matrix(c(1, r, r, 1), 2, 2), nw = 100000, np = 100000)
+		if(any(is.na(object@skewness))) {
+			Mvdc <- mvdc(object@copula, object@margins, object@paramMargins)
+			Data <- CopSEM(Mvdc, matrix(c(1, r, r, 1), 2, 2), nw = 100000, np = 100000)
+		} else {
+			Data <- dataGen(object, n = 100000, m = rep(0, 2), cm = matrix(c(1, r, r, 1), 2, 2))
+		}
 		obj <- find2Dhist(Data[,1], Data[,2], gridsize = c(500L, 500L))
 		xis <- obj$x1
 		yis <- obj$x2
@@ -83,7 +89,7 @@ plotDist2D <- function(distName, param, xlim = NULL, ylim = NULL, r = 0, reverse
 		yis <- yis[usedy]
 		zmat <- zmat[usedx, usedy]
 	}
-    if (reverse[1]) {
+    if (object@reverse[1]) {
         zmat <- zmat[nrow(zmat):1, ]
         den <- apply(zmat, 1, sum)
         wMeanOld <- sum(xis * den)/sum(den)
@@ -93,7 +99,7 @@ plotDist2D <- function(distName, param, xlim = NULL, ylim = NULL, r = 0, reverse
         wMeanNew <- sum(xis * den)/sum(den)
         xis <- seq(wMeanNew - disRightOld, wMeanNew + disLeftOld, length.out = length(xis))
     }
-    if (reverse[2]) {
+    if (object@reverse[2]) {
         zmat <- zmat[, ncol(zmat):1]
         den <- apply(zmat, 2, sum)
         wMeanOld <- sum(yis * den)/sum(den)
